@@ -1,21 +1,30 @@
 /* -*-mode:java; c-basic-offset:2; -*- */
-/* JZlib -- zlib in pure Java
- *  
- * Copyright (C) 2000 ymnk, JCraft, Inc.
- *   
- * This program is free software; you can redistribute it and/or
- * modify it under the terms of the GNU Library General Public License
- * as published by the Free Software Foundation; either version 2 of
- * the License, or (at your option) any later version.
- *   
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU Library General Public License for more details.
- * 
- * You should have received a copy of the GNU Library General Public
- * License along with this program; if not, write to the Free Software
- * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+/*
+Copyright (c) 2000,2001,2002,2003 ymnk, JCraft,Inc. All rights reserved.
+
+Redistribution and use in source and binary forms, with or without
+modification, are permitted provided that the following conditions are met:
+
+  1. Redistributions of source code must retain the above copyright notice,
+     this list of conditions and the following disclaimer.
+
+  2. Redistributions in binary form must reproduce the above copyright 
+     notice, this list of conditions and the following disclaimer in 
+     the documentation and/or other materials provided with the distribution.
+
+  3. The names of the authors may not be used to endorse or promote products
+     derived from this software without specific prior written permission.
+
+THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED WARRANTIES,
+INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
+FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL JCRAFT,
+INC. OR ANY CONTRIBUTORS TO THIS SOFTWARE BE LIABLE FOR ANY DIRECT, INDIRECT,
+INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
+LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA,
+OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
+LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
+NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
+EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  */
 /*
  * This program is based on zlib-1.1.3, so all credit should go authors
@@ -394,8 +403,10 @@ final class Deflate{
   }
 
   static boolean smaller(short[] tree, int n, int m, byte[] depth){
-    return (tree[n*2] < tree[m*2] ||
-	    (tree[n*2] == tree[m*2] && depth[n] <= depth[m]));
+    short tn2=tree[n*2];
+    short tm2=tree[m*2];
+    return (tn2<tm2 ||
+	    (tn2==tm2 && depth[n] <= depth[m]));
   }
 
   // Scan a literal or distance tree to determine the frequencies of the codes
@@ -559,7 +570,8 @@ final class Deflate{
   }   
 
   final void send_code(int c, short[] tree){
-    send_bits((tree[c*2]&0xffff), (tree[c*2+1]&0xffff));
+    int c2=c*2;
+    send_bits((tree[c2]&0xffff), (tree[c2+1]&0xffff));
   }
 
   void send_bits(int value, int length){
@@ -1019,7 +1031,9 @@ final class Deflate{
       // Find the longest match, discarding those <= prev_length.
       // At this point we have always match_length < MIN_MATCH
 
-      if(hash_head!=0L && strstart-hash_head<=w_size-MIN_LOOKAHEAD) {
+      if(hash_head!=0L && 
+	 ((strstart-hash_head)&0xffff) <= w_size-MIN_LOOKAHEAD
+	 ){
 	// To simplify the code, we prevent matches with the string
 	// of window index 0 (in particular we have to avoid a match
 	// of the string with itself at the start of the input file).
@@ -1126,7 +1140,8 @@ final class Deflate{
       match_length = MIN_MATCH-1;
 
       if (hash_head != 0 && prev_length < max_lazy_match &&
-	  strstart - hash_head <= w_size-MIN_LOOKAHEAD) {
+	  ((strstart-hash_head)&0xffff) <= w_size-MIN_LOOKAHEAD
+	  ){
 	// To simplify the code, we prevent matches with the string
 	// of window index 0 (in particular we have to avoid a match
 	// of the string with itself at the start of the input file).
@@ -1254,7 +1269,6 @@ final class Deflate{
 
       // Skip to next match if the match length cannot increase
       // or if the match length is less than 2:
-
       if (window[match+best_len]   != scan_end  ||
 	  window[match+best_len-1] != scan_end1 ||
 	  window[match]       != window[scan]     ||
@@ -1265,12 +1279,10 @@ final class Deflate{
       // It is not necessary to compare scan[2] and match[2] since they
       // are always equal when the other bytes match, given that
       // the hash keys are equal and that HASH_BITS >= 8.
-
       scan += 2; match++;
 
       // We check for insufficient lookahead only every 8th comparison;
       // the 256th check will be made at strstart+258.
-
       do {
       } while (window[++scan] == window[++match] &&
 	       window[++scan] == window[++match] &&
@@ -1292,6 +1304,7 @@ final class Deflate{
 	scan_end1  = window[scan+best_len-1];
 	scan_end   = window[scan+best_len];
       }
+
     } while ((cur_match = (prev[cur_match & wmask]&0xffff)) > limit
 	     && --chain_length != 0);
 
